-----------------------------------------------------------------------------

local tablet = AceLibrary("Tablet-2.0")
local L = AceLibrary("AceLocale-2.2"):new("FuBar_LootTypeFu")

-----------------------------------------------------------------------------

LootTypeFu = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "FuBarPlugin-2.0")
LootTypeFu:RegisterDB("LootTypeFuDB")
LootTypeFu:RegisterDefaults('profile', {
	showThresholdTooltip		= true,
	showMasterLooterTooltip		= true,
	showMasterLooter			= true,
	showMasterLooterNameOnly	= false,
	showExtraLootThresholds		= false,
	showHint					= true,
})

LootTypeFu.version = "r" .. string.sub("$Revision: 41009 $", 12, -3)
LootTypeFu.date = string.sub("$Date: 2007-06-21 04:40:09 -0400 (Thu, 21 Jun 2007) $", 8, 17)
LootTypeFu.hasIcon = "Interface\\AddOns\\FuBar_LootTypeFu\\LootType.tga"
LootTypeFu.defaultPosition = 'LEFT'
LootTypeFu.hideWithoutStandby = true
LootTypeFu.independentProfile = true
LootTypeFu.clickableTooltip = true

-----------------------------------------------------------------------------

local function get(k) return LootTypeFu.db.profile[k] end
local function set(k, v) LootTypeFu.db.profile[k] = v LootTypeFu:UpdateDisplay() end

LootTypeFu.OnMenuRequest = 
{
	handler = LootTypeFu,
	type = 'group',
	args = 
	{
		masterName =
		{
			type = "toggle",
			name = L["Show Master Looter's Name"],
			desc = L["Show the master looter's name on FuBar"],
			get = get,
			set = set,
			passValue = "showMasterLooter",
			order = 1,
		},
		masterLootType =
		{
			type = "toggle",
			name = L["  Show Name Only"],
			desc = L["Remove the Master Looter prefix"],
			disabled = function() return not LootTypeFu.db.profile.showMasterLooter end,
			get = get,
			set = set,
			passValue = "showMasterLooterNameOnly",
			order = 2,
		},
		tooltip =
		{
			type = "group",
			name = L["Tooltip Options"],
			desc = L["Options for LootTypeFu's tooltip"],
			args =
			{
				showThreshold =
				{
					type = "toggle",
					name = L["Show Threshold"],
					desc = L["Show loot threshold on tooltip"],
					get = get,
					set = set,
					passValue = "showThresholdTooltip",
					order = 1,
				},
				showExtraThresholds =
				{
					type = "toggle",
					name = L["  Show Extra Thresholds"],
					desc = L["Shows Legendary and Artifact loot threshold options"],
					disabled = function() return not LootTypeFu.db.profile.showThresholdTooltip end,
					get = get,
					set = set,
					passValue = "showExtraLootThresholds",
					order = 2,
				},
				showMasterLooter =
				{
					type = "toggle",
					name = L["Show Master Looter"],
					desc = L["Show master looter on tooltip"],
					get = get,
					set = set,
					passValue = "showMasterLooterTooltip",
					order = 3,
				},
				showHint =
				{
					type = "toggle",
					name = L["Show Hint"],
					desc = L["Show hint on tooltip"],
					get = get,
					set = set,
					passValue = "showHint",
					order = 4,
				},
			}
		}
	},
}

-----------------------------------------------------------------------------

function LootTypeFu:OnInitialize()
end

function LootTypeFu:OnEnable()
    self:RegisterEvent("PARTY_MEMBERS_CHANGED", "UpdateDisplay")
    self:RegisterEvent("RAID_ROSTER_UPDATE", "UpdateDisplay")
    self:RegisterEvent("PARTY_LOOT_METHOD_CHANGED", "UpdateDisplay")
end

function LootTypeFu:OnDisable()
end

-----------------------------------------------------------------------------

function LootTypeFu:OnTextUpdate()
	if GetNumPartyMembers() + GetNumRaidMembers() > 0 then
		local lootMethod, lootMaster = GetLootMethod()
		local lootMasterName		
		local lootText
		if lootMethod == "master" and self.db.profile.showMasterLooter then
			if lootMaster then
				lootMasterName = UnitName( lootMaster == 0 and "player" or ("party"..lootMaster) )
			else
				for i=1,GetNumRaidMembers() do
					if select(11, GetRaidRosterInfo(i)) then
						lootMasterName = UnitName("raid"..i)
						break
					end
				end
			end
			if lootMasterName then
				if self.db.profile.showMasterLooterNameOnly then
					lootText = lootMasterName
				else
					lootText = L[lootMethod].."|r: "..lootMasterName
				end
			else
				lootText = L[lootMethod]			
			end
		else
			lootText = L[lootMethod]
		end
		local color = ITEM_QUALITY_COLORS[GetLootThreshold()].hex
		self:SetText(color..lootText)
	else
		self:SetText(L["Inactive"])
	end
end

-----------------------------------------------------------------------------

function LootTypeFu:OnClickLootType(lootType)
	SetLootMethod(lootType, UnitName("player"))
end

function LootTypeFu:OnClickLootThreshold(threshold)
	SetLootThreshold(threshold)
end

function LootTypeFu:OnClickMasterLooter(name)
	SetLootMethod("master", name)
end

-----------------------------------------------------------------------------

local lootTypeList =
{
	freeforall		= true,
	roundrobin		= true,
	master			= true,
	group			= true,
	needbeforegreed = true,
}

local temporaryTable = { }
local unknownColor = { r = 0.5, g = 0.5, b = 0.5 }

function LootTypeFu:OnTooltipUpdate()
	if (GetNumPartyMembers() > 0) or (GetNumRaidMembers() > 0) then

		local canChangeLoot = (GetNumPartyMembers() + GetNumRaidMembers() > 0) and IsPartyLeader()
		local lootMethod = GetLootMethod()
		
		tablet:SetTitle(L["Loot Type"])
		local cat = tablet:AddCategory('text',L["Loot Mode"])
		for k in pairs(lootTypeList) do
			cat:AddLine(
				'text', L[k],
				'textR', not canChangeLoot and 0.6 or nil,
				'textG', not canChangeLoot and 0.6 or nil,
				'textB', not canChangeLoot and 0.6 or nil,
				'hasCheck', true,
				'checked', k == lootMethod,
				'func', canChangeLoot and 'OnClickLootType' or nil,
				'arg1', self,
				'arg2', k
			)
		end
		
		if (lootMethod == "group" or lootMethod == "needbeforegreed" or lootMethod == "master") and self.db.profile.showThresholdTooltip then
			-- Show threshold options
			local cat = tablet:AddCategory('text',L["Loot Threshold"])
			
			for i = 2, self.db.profile.showExtraLootThresholds and 6 or 4 do
				cat:AddLine(
					'text', _G["ITEM_QUALITY"..i.."_DESC"],
					'textR', ITEM_QUALITY_COLORS[i].r,
					'textG', ITEM_QUALITY_COLORS[i].g,
					'textB', ITEM_QUALITY_COLORS[i].b,
					'hasCheck', true,
					'checked', i == GetLootThreshold(),
					'func', canChangeLoot and 'OnClickLootThreshold' or nil,
					'arg1', self,
					'arg2', i
				)			
			end
		end
		
		if lootMethod == "master" and self.db.profile.showMasterLooterTooltip then
			local cat = tablet:AddCategory('text', L["Master Looter"])
			local masterLooter

			if GetNumRaidMembers() == 0 then
				_, masterLooter = GetLootMethod()
				if masterLooter then
					if masterLooter == 0 then
						masterLooter = "player"
					else
						masterLooter = "party"..masterLooter
					end
				end

				table.insert(temporaryTable, "player")
				for i = 1,GetNumPartyMembers() do
					table.insert(temporaryTable, "party"..i)
				end
				
			else
				for i=1,GetNumRaidMembers() do
					local unitid = "raid"..i
					table.insert(temporaryTable, unitid)
					
					-- IsMasterLooter check
					if select(11, GetRaidRosterInfo(i)) then masterLooter = unitid end
				end
			end

			table.sort(temporaryTable, function(a, b) return UnitName(a) < UnitName(b) end)

			for i,k in ipairs(temporaryTable) do
				local _, class = UnitClass(k)
				local color = RAID_CLASS_COLORS[class] or unknownColor
			
				cat:AddLine(
					'text', UnitName(k),
					'textR', color.r,
					'textG', color.g,
					'textB', color.b,
					'hasCheck', true,
					'checked', k == masterLooter,
					'func', canChangeLoot and 'OnClickMasterLooter' or nil,
					'arg1', self,
					'arg2', UnitName(k)
				)
				
				temporaryTable[i] = nil
			end
		end

		if self.db.profile.showHint then
			tablet:SetHint(L["TOOLTIP_HINT"])
		end
	end
end

-----------------------------------------------------------------------------
